function khi=StatTerM(bisp,n,T,winsize,overlap)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB code
% File: StatTerM.m
% Coded by Gy. Terdik and J. M\'{a}th
%
% Computes the Terdik-Math test statistic.
%
% bisp    = the estimated bispectrum on the lattice (i/T, j/T, i,j=1:T-1)
% n       = the length of the time series
% T       = the length of a strech
% winsize = the size of window used in the bispectrum estimation
% overlap = the size of overlapping in the bispectrum estimation
%
% Reference:
% Terdik, G. and M\'{a}th, J. (1998).
%    A new test of linearity of time series based on the bispectrum.
%    Journal of Time Series Analysis, 19(6), 737-753.
%    DOI: 10.1111/1467-9892.00120.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

TBT2=winsize^2/T;
K=fix((n-T*overlap/100)/T/(100-overlap)*100); %%% number of stretches,

%%%%%%%%%%%%%%%%%%%   Triplets %%%%%%%%
IND=T/64* [17 27 30
           17 21 10
           17 24 27
           18 27 14
           18 21 24
           19 30  1
           21 27  9];        

for k=1:7;
   kip=[IND(k,1) IND(k,2)
       IND(k,3)  0
       -IND(k,1)+IND(k,3),  -IND(k,2)-IND(k,3)];
   kin=[IND(k,2) IND(k,3)
       0, -IND(k,1)-IND(k,2)
       -IND(k,1)+IND(k,3), -IND(k,3)];
      for j=1:2
         for m=1:3
           if kip(m,j)<0
              kip(m,j) = kip(m,j)+T;
           end;
           if kin(m,j)<0
              kin(m,j) = kin(m,j)+T;
           end;
         end;
       end;
%%%%%%%%%%%%%%% kip kin containes the 6 points of the lattice necessary
%%%%%%%%%%%%%%% to compute the test statistics

   bp    = bisp(kip(1,1)+1,kip(1,2)+1)+bisp(kip(2,1)+1,kip(2,2)+1)+ ...
           bisp(kip(3,1)+1,kip(3,2)+1);
   bn    = bisp(kin(1,1)+1,kin(1,2)+1)+bisp(kin(2,1)+1,kin(2,2)+1)+ ...
           bisp(kin(3,1)+1,kin(3,2)+1);
   st(k) = bp-bn;
end;     % end loop for k=1:7

%%%%%%%%%%%%% Test statistic 
                                      %%%               T,K
khi = K*TBT2/1.4628*norm(st,2)^2/3;   %%% The value of G
                                      %%%               n

