%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MATLAB code: Exercise 4.4 (optional)
% File: simulation_tests.m
%    
% OUTPUT:  ``Testsall'' a (20 * 3) matrix with percentage rejections of 
%          three test statistics at a 5% nominal level.
%          [Optional output: test results for IQR (estimated and theoretical)
%           test results for Lambda linearity test statistic.]
%
% Columns:
%   1) Hinich's Gaussianity (HG) test statistic 
%   2) Hinich's linearity (HL) test statistic 
%   3) Terdik-Math linearity test statistic 
% 
% Rows:
%   1) NID(0,1)    (Gaussian and linear; Berg et al., and Jahan/Harvill);
%   2) Chi^{2}(1)  (not Gaussian but linear; Berg et al.);
%   3) AR(1)       (Gaussian and linear; Berg et al.);
%   4) ARCH(1)     (neither Gaussian nor linear);
%   5) TGARCH      (neither Gaussian nor linear);
%   6) Bilinear    (neither Gaussian nor linear);
%   7) Nonl MA     (neither Gaussian nor linear);
%   8) TAR         (neither Gaussian nor linear);
%   9) FAR(1)      (neither Gaussian nor linear); 
%  10) Sign AR(1)  (neither Gaussian nor linear);     
%
% Additional models considered by Berg et al. (2010): 
%  11) AR(1)        
%  12) ARMA(2,2)    
%  13) Bilinear(1,0,1,1)     
%  14) ARCH(1) 
%
% Additional models considered by Jahan and Harvill (2008):
%  15) AR(2); 
%  16) MA(2); 
%  17) Nonl MA; 
%  18) BL; 
%  19) TAR; 
%  20) Nonl AR(2) 
%
% References:
% Berg, A., Paparoditis, E., and Politis, D.N. (2010).
%    A bootstrap test for time series linearity.
%    Journal of Statistical Planning and Inference, 140(12), 3841-3857.
%    DOI: 10.1016/j.jspi.2010.04.047.
% Jahan, N. and Harvill, J.L. (2008).
%    Bispectral-based goodness-of-fit tests of Gaussianity
%    and linearity of stationary time series.
%    Communications in Statistics: Theory and Methods, 37(20), 3216-3227.
%    DOI: 10.1080/03610920802133319.
% Terdik, G. and M\'{a}th, J. (1998).
%    A new test of linearity of time series based on the bispectrum.
%    Journal of Time Series Analysis, 19(6), 737-753.
%    DOI: 10.1111/1467-9892.00120.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
warning('off','MATLAB:dispatcher:InexactCaseMatch')

n    = 2^8;   % Length of the simulate time series, e.g., n = 256 = 2^8, 
              % n = 512 = 2^9, and n = 1024 = 2^10
cikl = 1000;  % Number of MC replications
T    = 2^6;   % Length of a stretch
fed  = 0;     % Overlaping
nm   = 20;    % Number of models selected
%-------------------------------------------------------------------------------
window=16;     % T*bT, size of window during the bispectum estimation

rng('default') % The same random numbers are produced as if you
               % restarted MATLAB

for jj=1:cikl;    % Start loop over replications
   x  = zeros(n+100,1);
   w1 = normrnd(0,1,n+100,1);

   xx      = zeros(20,n);               % 1) WN iid (0,1)
   www     = w1(101:n+100,1);      
   xx(1,:) = www';
   
   w2      = chi2rnd(1,n+100,1);        % 2) chi^{2}_1 
   www     = w2(101:n+100,1); 
   xx(2,:) = www';
                           
   nsize = n;
   y      = normrnd(0,1,nsize+100,1);  
   x(1,1) = 0;                          % 3)  AR(1) model
   for j=2:nsize+100
       x(j,1) = 0.3*x(j-1,1)+y(j,1);                    
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(3,:) = z';

   x(1,1) = 0.0;
   for j=2:nsize+100
       x(j,1) = y(j,1)*sqrt(1.0+0.9*x(j-1,1)*x(j-1,1)); % 4) ARCH(1) model
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(4,:) = z';

   ee1 = normrnd(0,1,nsize+100,1);                      % 5) Threshold GARCH
   ee2 = normrnd(0,1,nsize+100,1);
   sigma(1,1) = 0.0;
     
   for j=2:nsize+100
    if(ee1(j-1,1)<0)
      sigma(j,1) = sqrt(0.25+0.6*sigma(j-1,1)*sigma(j-1,1)+...
                   0.5*ee1(j-1,1)*ee1(j-1,1));
    else 
      sigma(j,1) = sqrt(0.25+0.6*sigma(j-1,1)*sigma(j-1,1)+...
                   0.2*ee1(j-1,1)*ee1(j-1,1));
    end
      ee1(j,1) = ee2(j,1)*sigma(j,1);
   end
   for j=1:nsize
       z(j,1) = ee1(j+100,1);
   end
   xx(5,:) = z';
  
   x(1,1) = 0.0;
   for j=2:nsize+100
       x(j,1) = 0.8*x(j-1,1)*y(j-1,1)+y(j,1); % 6) Bilinear AR(1) model
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(6,:) = z';

   x(1,1) = 0.0;                               % 7) Nonlinear MA
   for j=2:nsize+100
       x(j,1) = 0.8*y(j-1,1)^2+y(j,1);
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(7,:) = z';   

   x(1,1) = 0;
   for j=2:nsize+100                           % 8) Threshold AR
      x(j,1) = -0.5.*x(j-1,1)*(x(j-1,1)<=1)+0.4*x(j-1,1)*(x(j-1,1)>1)+y(j,1); 
   end                                             
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(8,:) = z';   

   x(1,1) = 0;         
   for j=2:nsize+100    
      x(j,1) = 0.8*(abs(x(j-1,1)))^0.5+y(j,1);  % 9) Fractional AR(1)
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(9,:) = z';
            
   x(1,1) = 0;                                 
   for j=2:nsize+100
      x(j,1) = (2.0*(x(j-1,1)>0)-1)+0.43*y(j,1); % 10) Sign AR(1)
   end
   for j=1:nsize
      z(j,1) = x(j+100,1);
   end
   xx(10,:) = z';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Berg's set of models (NB: Their models 1) and 2) are
%  identical to models 1) and 2) above.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   x(1,1) = 0;                             % 11) AR(1) model
   for j=2:nsize+100
        x(j,1) = 0.9*x(j-1,1)+y(j,1);                    
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(11,:) = z';

   yy = normrnd(0,0.1796,nsize+100,1);        
   x(1,1) = 0;               % 12) ARMA(2,2) model with N(0,0.1796^2) WN
   x(2,1) = 0;
   for j=3:nsize+100
      x(j,1) = 0.8897*x(j-1,1)-0.4858*x(j-2,1)+...
               yy(j,1)-0.2279*yy(j-1,1)+0.2488*yy(j-2,1);                    
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(12,:) = z';

   x(1,1) = 0.0;
   for j=2:nsize+100
       x(j,1) = 0.4*x(j-1,1)+0.4*x(j-1,1)*y(j-1,1)+y(j,1); % 13) Bil AR(1) model
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(13,:) = z';
    
   x(1,1) = 0.0;
   for j=2:nsize+100
        x(j,1) = y(j,1)*sqrt(1.5+0.9*x(j-1,1)*x(j-1,1));   % 14) ARCH(1) model
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(14,:) = z';
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  Jahan & Harvill's set of models (Note: Their model I is 
%  identical to model 1) above.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
   x(1,1) = 0;                             % 15)  AR(2) model
   x(2,1) = 0;
   for j=3:nsize+100
        x(j,1) = 0.4*x(j-1,1)-0.3*x(j-2,1)+y(j,1);                    
   end
   for j=1:nsize
       z(j,1) = x(j+100,1);
   end
   xx(15,:) = z'; 
    
   x(1,1) = 0;                             % 16)  MA(2) model
   x(2,1) = 0;
   for j=3:nsize+100
        x(j,1) = y(j,1)-0.4*y(j-1,1)+0.3*y(j-2,1);                    
    end
    for j=1:nsize
       z(j,1) = x(j+100,1);
    end
    xx(16,:) = z';
    
    x(1,1) = 0.0;                          % 17)  nonlinear MA
    for j=2:nsize+100
       x(j,1) = -0.6*y(j-1,1)^2+0.5*y(j-1,1)+y(j,1);
    end
    for j=1:nsize
       z(j,1) = x(j+100,1);
    end
    xx(17,:) = z';
    
    x(1,1) = 0.0;
    for j=2:nsize+100
       x(j,1) = 0.4*x(j-1,1)-0.3*x(j-1,1)+0.8*y(j-1,1)+...
                0.5*x(j-1,1)*y(j-1,1)+y(j,1); 
    end                                    % 18) Bil AR(1) model
    for j=1:nsize
       z(j,1) = x(j+100,1);
    end
    xx(18,:) = z'; 
       
    x(1,1) = 0;
    for j=2:nsize+100
       x(j,1) = (1-0.5.*x(j-1,1))*(x(j-1,1)<0)+...
                (-1-0.5*x(j-1,1))*(x(j-1,1)>=0)+y(j,1);  
    end                                             
    for j=1:nsize                           % 19) Threshold AR(1)
       z(j,1) = x(j+100,1);
    end
    xx(19,:) = z';       
        
    x(1,1) = 0;                              % 20) Nonlin AR(2) model
    x(2,1) = 0;
    for j=3:nsize+100
        x(j,1) = -0.25*x(j-1,1)+0.2*x(j-2,1)+...
                  0.15*x(j-1,1)*x(j-1,1)-0.1*x(j-2,1)*x(j-2,1)+y(j,1);                    
    end
    for j=1:nsize
       z(j,1) = x(j+100,1);
    end
    xx(20,:) = z';     
    
  for k=1:nm;                                    % Start loop over models
    [w,A,C,SBC,FPE,th] = arfit(xx(k,:)',1,10);   % Maximum AR order is 10 
    [m,order] = size(A);
    order     = order+1;
    ord(jj,k) = order;

    e    = filter(A,1,xx(k,:)');                  
    stde = std(e);
    e    = e/stde;
   
    bisp = bispwin(e,T,window, fed); % bispwin contains the estimated bispectrum
                                     % on the lattice (i/T, j/T, i,j=1:T-1)

    chi_val = StatTerM(bisp,n,T,window,fed); % Value of the Terdik-Math statistic

    pvalchi        = 1-chi2cdf(chi_val,14);  % p-value
    pvalTerM(jj,k) = pvalchi;           
 
    chivv(jj,k)    = chi_val;
    [sg,sl]        = glstat2(xx(k,:)'); % glstat2 = modified version of glstat,
                                        % suppressing output

    psg              = sg(:,3);
    pvalHG(jj,k)     = psg;             % p-value Hinich's Gaussianity test
 
    testline         = sl(:,1);
    testlinemp(jj,k) = testline;        % R-75% range (estimated value)
 
    testlbd          = sl(:,2);
    testlambda(jj,k) = testlbd;         % Lambda linearity test
 
    testlinth        = sl(:,3);
    testlinthe(jj,k) = testlinth;       % R-75% range (theoretical value linearity test)
 
    pvallin          = sl(:,4);
    pvalHL(jj,k)     = pvallin;         % p-value Hinich's linearity test
 
  end  %  end loop over models k=1:nm    
end    %  end loop over replication jj=1:cikl  

%% clear testlinth testlbd testline psg pvallin pvalchi 
%% clear chi_val order A K FPE y z th n j yy
%% clear SBC a1 a2 e e1 e2 C T ee1 ee2 fed i jj k m w1 w2 
%% clear sl sg sigma window x xx w www stde

for k=1:nm
    sum = 0;
    for i=1:cikl
        if pvalTerM(i,k)<0.05        % 5% significance level
           sum = sum+1;
        end
        TerMLinear(1,k) = sum/cikl;  % percentage rejections of linearity hypothesis
    end                            
end

for k=1:nm
    sum = 0;
    for i=1:cikl
        if pvalHG(i,k)<0.05 
           sum = sum+1;
        end
        HinichGauss(1,k) = sum/cikl;  % percentage rejections of Gaussianity hypothesis
    end
end

for k=1:nm
    sum = 0;
    for i=1:cikl
        if pvalHL(i,k)<0.05 
           sum = sum+1;
        end
        HinichLinear(1,k) = sum/cikl;  % percentage rejections of linearity hypothesis
    end
end

Testsall = [HinichGauss ; HinichLinear ; TerMLinear ]';

